/*
 * SunGridClient.java
 *
 * Created on Dec.12, 2007, 2:12 PM, 
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.monash.sungrid;

import com.sun.sungrid.gridclient.GridConnection;
import com.sun.sungrid.gridclient.GridException;
import com.sun.sungrid.gridclient.GridSession;
import com.sun.sungrid.gridclient.Job;
import com.sun.sungrid.gridclient.JobHandler;
import com.sun.sungrid.gridclient.JobResponse;
import com.sun.sungrid.gridclient.Resource;
import com.sun.sungrid.gridclient.ResourceHandler;
import com.sun.sungrid.gridclient.ResourceResponse;
import com.sun.sungrid.gridclient.Run;
import com.sun.sungrid.gridclient.RunHandler;
import com.sun.sungrid.gridclient.RunResponse;
import com.sun.sungrid.gridclient.SessionResponse;
import com.sun.sungrid.gridclient.UserIdentity;
import java.io.File;
import java.util.List;
import java.util.Arrays;
import javax.swing.Icon;
import javax.swing.ImageIcon;
import javax.swing.JLabel;
import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.gui.style.TextStyle;
import ptolemy.data.ArrayToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.data.IntToken;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

import javax.swing.JOptionPane;
import javax.swing.JPasswordField;
import javax.swing.JTextField;


/**
  *   <p> SunGridClient.java is a subclass of ptolemie's TypedAtomicor.
  *   The actor is used to define a job which can run on Sun Grid. 
  *   The configuration of job is defined by SunGridClient Actor, which include 
  *   1) Job Name, which is used to identify jobs on SunGrid,
  *      It should be unique charactor for job
  *   2) Job Description
  *   3) Job Notes
  *   4) Executable, it defines the executeable program or script file which will be initially invoked for the job 
  *   5) Parameters, the exexutable parameters, it's listed as a string and separated by comma
  *   6) Output File, each job running on Sun Grid will get output file which contians the running status and 
  *      all the output from the application, so users can specify the file name for compressed output
  *
  *   <p>To access Sun Grid(network.com), there are three important actors
  *     SunGridLogin            --  Setting Login info, which include user name and proxy 
  *     SunGridInputResource    --  Define resource files for Sun Grid job, and will be upload to Sun Grid before job running
  *                                 One Sun Grid job can link with several SunGridInputResource
  *     SunGridClient           --  Define Sun Grid job which is used to run application on Sun Grid</p>
  *
  *    
  * @author Chris Zhu, Sun Microsystems, Inc.
  * @version $Id: SunGridLogin.java, v 1.0 2007/12/12
  **/
public class SunGridClient  extends TypedAtomicActor {

   /** Creates a new instance of SunGridClient */
    public SunGridClient(CompositeEntity container, String name)
            throws NameDuplicationException, IllegalActionException  
    {
        super(container, name);
        
        //------------- initialize multi-input port -------------
        inputResource = new TypedIOPort(this, "Input Resource", true, false);
        inputResource.setTypeEquals(new ArrayType(BaseType.STRING));
        new Parameter(inputResource, "_showName", BooleanToken.TRUE);
        inputResource.setMultiport(true);
        
        gridLogin = new TypedIOPort(this, "Grid Login", true, false);
        gridLogin.setTypeEquals(new ArrayType(BaseType.STRING));
        new Parameter(gridLogin, "_showName", BooleanToken.TRUE);

        outputResource = new TypedIOPort(this, "Output Resource", false, true);
        outputResource.setTypeEquals(new ArrayType(BaseType.STRING));
        new Parameter(outputResource, "_showName", BooleanToken.TRUE);        
        
        outputMonitor = new TypedIOPort(this, "Output Monitor", false, true);
        outputMonitor.setTypeEquals(new ArrayType(BaseType.STRING));
        new Parameter(outputMonitor, "_showName", BooleanToken.TRUE);    
        
        //------------- initialize parameters -------------
        // job Name  -- must be unique defined
        // can't be null
        jobName = new Parameter(this, "Job Name", new StringToken("Can't be null"));
        jobName.setStringMode(true);
 
        // initialize a one line job description 
        jobDescription = new Parameter(this, "Job Description", new StringToken("Enter Description here..."));
        jobDescription.setStringMode(true);
        
        // initialize a multiple line job notes 
        jobNotes = new Parameter(this, "JobNotes", new StringToken(""));
        // make a text style, so that the parameter has multiple lines
        new TextStyle(jobNotes, "Job Notes");
	// set the default script        
	jobNotes.setExpression("You can add your notes here...");
        jobNotes.setStringMode(true);

        // executeable program/script file which will be initially invoked for the job 
        // Can't be null
        jobExecutable = new Parameter(this, "Job Executable", new StringToken("Enter executeable program/script file here (start of the job)"));
        jobExecutable.setStringMode(true);

        // job parameters listed as a string and separated by comma
        jobParameters = new Parameter(this, "Job Parameters", new StringToken("You can add runtime arguments or parameters here, if any..."));
        jobParameters.setStringMode(true);
        
        // file name for compressed output 
        outputFile = new Parameter(this, "Output File",new StringToken("Enter output entire file name or path here") );
        outputFile.setStringMode(true);

        // timeout of the job 
        timeout = new Parameter(this, "Timeout",new IntToken("-1") );
      //  timeout.set

	_attachText("_iconDescription", "<svg>\n"
			+ "<rect x=\"-25\" y=\"-20\" " + "width=\"50\" height=\"40\" "
			+ "style=\"fill:white\"/>\n"
			+ "<polygon points=\"-15,-10 -12,-10 -8,-14 -1,-14 3,-10"
			+ " 15,-10 15,10, -15,10\" " + "style=\"fill:yellow\"/>\n"
			+ "</svg>\n");   
        
        runID = null;
        trustStore = null;
    }

    static GridSession session;
    private String userName; 
    
    // multi-input resources for SunGridClient
    public TypedIOPort inputResource;
    // login info for SunGridClient
    public TypedIOPort gridLogin;

    // output file of the job 
    public TypedIOPort outputResource;
    // the output info when the job is finished runnig
    public TypedIOPort outputMonitor;
    
    // resource name -- unique charactor for resource
    public Parameter jobName;
    // resource description
    public Parameter jobDescription;
    // resource notes
    public Parameter jobNotes;
    // the name of executable file initially invoked for the job
    public Parameter jobExecutable;
    // runtime arguments or parameters, if any, for the initial executable
    public Parameter jobParameters;
    // output resource file
    public Parameter outputFile;
    // timeout of the job
    public Parameter timeout;
    
    
    private String runID;
    private String trustStore;

    private boolean InitialLoginInfo(Token loginToken) 
            throws IllegalActionException 
    {
        ArrayToken token = (ArrayToken) loginToken;
        if ( token.length() != 6)
        {
            JOptionPane.showMessageDialog(null, 
                    "Please check the SunGridLogin Actor.", 
                    "Login Error",
                    JOptionPane.ERROR_MESSAGE); 
            return false;
        }

        Token[] para = token.arrayValue();
        userName = ((StringToken)para[0]).stringValue();
        System.setProperty("gridclient.internalAPI.proxyHost", ((StringToken)para[1]).stringValue());
        System.setProperty("gridclient.internalAPI.proxyPort", ((StringToken)para[2]).stringValue());
        System.setProperty("gridclient.internalAPI.url", ((StringToken)para[3]).stringValue());
        System.setProperty("gridclient.adapterName", ((StringToken)para[4]).stringValue());
        System.setProperty("gridclient.internalAPI.connection.class", ((StringToken)para[5]).stringValue());
        return true;
     }    
    
    public ResponseInfo SunGridConnect(String username, String password)    
    {
        ResponseInfo rel = null;
        String errorMessage;
        if(null == username)
        {
            rel = new ResponseInfo(false, "No user name");
            return rel;
        } else if(null == password)
        {
            rel = new ResponseInfo(false, "Password is NULL");
            return rel;
        }
        else
        {
            SessionResponse sessionResponse = null;
            try
            {
                // Get a connection to the Grid
                GridConnection connection = GridConnection.createConnection();
                connection.connect();

                // create a session and login
                session = connection.createSession();
                
                if (_debugging) 
                {            
                    char[] obfuscatedPwdArray = new char[password.length()];
                    Arrays.fill(obfuscatedPwdArray, '*');
                    String obfuscatedPwd = new String(obfuscatedPwdArray);
                    _debug("Connecting to '" + System.getProperty("gridclient.internalAPI.url") + "' by - username '" + username + "' - password '" + obfuscatedPwd + "'\n");
                    _debug("Proxy is '" + System.getProperty("gridclient.internalAPI.proxyHost") + ":" + System.getProperty("gridclient.internalAPI.proxyPort") + "'\n");
                }
                
                sessionResponse = session.loginAndWait(new UserIdentity(username, password));
                if(sessionResponse.isSuccessfull())
                    return new ResponseInfo(true);
                rel = new ResponseInfo(false, sessionResponse.getDescription());
                return rel;
             }
            catch (GridException ex) {
                rel = new ResponseInfo(false, ex.getMessage());
                System.out.println("The exception raised the message " + ex.getMessage()); 
                ex.printStackTrace();
                return rel;
             } 
        }
    }
    
    public boolean SunGridLogin() 
    {
        String password = null;
        int result;
        
        // UI for enter the password 
        JTextField passwordField = (JTextField) new JPasswordField(20);
        
        String path = System.getProperty("KEPLER") + "/src/org/monash/sungrid/Icon/sungrid.gif";
        Object[] ob = { new JLabel("\nAccount Login"), new JLabel("User Name :  " + userName) , new JLabel("Passowrd  :  "), passwordField };
        result = JOptionPane.showConfirmDialog(null, 
                    ob, 
                    "Welcome to Sun Grid",
                    JOptionPane.OK_CANCEL_OPTION,
                    JOptionPane.QUESTION_MESSAGE,
                    (Icon)(new ImageIcon(path)));
        
        if (result == JOptionPane.OK_OPTION) {
            password = passwordField.getText();
        } 
        else
        {
            return false;
        }

        // Login to Sun Grid
        ResponseInfo rev = SunGridConnect(userName, password);
        
        if(!rev.isSuccessfull())
        {
            result = JOptionPane.showConfirmDialog(null, 
                            "Error: " + rev.getDescription()  + "\nDo you want to login again ?", 
                            "Login Failed",
                            JOptionPane.OK_CANCEL_OPTION,
                            JOptionPane.WARNING_MESSAGE);
    
            if (result == JOptionPane.OK_OPTION) {
                return SunGridLogin();
            } else {
                if(null != session)
                    session.close();
                return false;
            }                                  
        }
        else
            return true;
    }
    
    private void inputResourceParse(Token inputToken, Resource newResource) 
            throws IllegalActionException 
    {
        try
        {
            ArrayToken token = (ArrayToken) inputToken;

            if ( token.length() != 5)
                   return ;               
  
            Token[] para = token.arrayValue();
            
            newResource.setName(((StringToken)para[0]).stringValue());
            newResource.setDescription(((StringToken)para[1]).stringValue());
            newResource.setType(((StringToken)para[2]).stringValue());
            newResource.setNotes(((StringToken)para[3]).stringValue());
            newResource.setFileName(((StringToken)para[4]).stringValue());
        }
        catch (Exception ex) {
            System.out.println("The exception raised in Parsing Resource Token with the message: " + ex.getMessage());  
            ex.printStackTrace();
        }
    }
        
    public ResponseInfo AddResource(Job job) 
            throws IllegalActionException 
    {
        ResponseInfo rel = null;
        ResourceHandler resourceHandler = session.getResourceHandler(); // get the resource handler

        Token inputToken;
        Resource newResource;
        int resource_num = inputResource.numLinks();
        for (int i = 0; i < resource_num; i++)
        {
            // Create a local resource instance attached to the session
            newResource = resourceHandler.createResource();

            inputToken = inputResource.get(i);
            inputResourceParse(inputToken, newResource);
                
            ResourceResponse resourceResponse;
            try
            {
                if (_debugging)
                    _debug("Try to delete the resource from Sun Grid if it has the same name as new resource\n");
                // Delete the resource from Sun Grid if it has the same name as new resource
                resourceResponse = resourceHandler.getResourceAndWait(newResource.getName());
                resourceResponse.getResources().get(0).deleteAndWait();
            }
            catch (GridException e)
            {} 

            try
            {
                 if (_debugging) {            
                     _debug("Begin to add new resource ------ " + newResource.getFileName());
                 }
  System.out.println("\n --------- Resource Information--------- " + 
                        "\n - Name '" + newResource.getName() +
                        "'\n - Description '" + newResource.getDescription() +
                        "'\n - Owner '" + newResource.getOwner() +
                        "'\n - Size '" + newResource.getSize() +
                        "'\n - FileName '" + newResource.getFileName() +
                        "'\n - Type '" + newResource.getType() +
                        "'\n");                
                // Add resouce to Sun Grid
                 resourceResponse = newResource.addAndWait(newResource.getFileName());
                 if (_debugging) {            
                     _debug("Add resource -" + newResource.getFileName() + "finished");
                 }
                 
                // Get the list of resources from the response
                List resources = resourceResponse.getResources();

                // Get the newly returned Resource instance that was added to the grid.  Since we
                // made the request we know there is only 1 resource in the list.
                newResource = (Resource)resources.get(0);
                
                // Print some information about the resource
                if(_debugging)
                {
                        _debug("\n --------- Resource Information--------- " + 
                        "\n - Name '" + newResource.getName() +
                        "'\n - Description '" + newResource.getDescription() +
                        "'\n - Owner '" + newResource.getOwner() +
                        "'\n - Size '" + newResource.getSize() +
                        "'\n - FileName '" + newResource.getFileName() +
                        "'\n - Type '" + newResource.getType() +
                        "'\n");
                }
                // Add Sun Grid resouce to the job
                job.addResource(newResource.getName());               
            }
            catch (GridException ex)
            {
                rel = new ResponseInfo(false, "Failed in adding resource " + newResource.getName() + " for job " + job.getName() + "\nError: " + ex.getMessage());
                ex.printStackTrace();
                return rel;
            } 
        }
        return new ResponseInfo(true);
    }
    
    public synchronized void fire() throws IllegalActionException 
    {
        String outStr="";
 
         try
        {
            try
            {
                Token loginToken = gridLogin.get(0);
                if(!InitialLoginInfo(loginToken))
                {
                      return;                         
                }
            }            
            catch (Exception ex)
            {
                 JOptionPane.showMessageDialog(null, 
                            "Please check the SunGridLogin Actor:\n" + ex.getMessage(), 
                            "Login Error",
                            JOptionPane.ERROR_MESSAGE);
                 ex.printStackTrace();
                 return;
            }            

            ////////////////////////////////////////////////////////////////////////////////////
            // login to Sun Grid
            if(!SunGridLogin())
            {
                return ;
            }

            ////////////////////////////////////////////////////////////////////////////////////
            // Add a new job to Sun Grid 
            // Get the job handler for the session
            JobHandler jobHandler = session.getJobHandler();

            // Create a local job attached to the session
            Job job = jobHandler.createJob();

            String job_name = ((StringToken)jobName.getToken()).stringValue();
            job.setName(job_name);
            job.setExecutable(((StringToken)jobExecutable.getToken()).stringValue());
            job.setParameters(((StringToken)jobParameters.getToken()).stringValue());
      
            // Add resources to the job
            ResponseInfo rev = AddResource(job);
            if(!rev.isSuccessfull())
            {
                JOptionPane.showMessageDialog(null, 
                            rev.getDescription(), 
                            "Resource Error",
                            JOptionPane.ERROR_MESSAGE);
    
                return;     
            }
            
            // Add the job to Sun Grid and wait for the response
            JobResponse jobResponse;
            try
            {
                if (_debugging) {            
                     _debug("Try to delete the job from Sun Grid if its name is as same as the new job");
                 }
                // Delete the job from Sun Grid if its name is as same as the new job
                jobResponse = jobHandler.getJobAndWait(job.getName());
                jobResponse.getJobs().get(0).deleteAndWait();
            }
            catch (GridException e)
            {}

            if (_debugging) {            
                     _debug("Begin to add job to Sun Grid");
            }
            jobResponse = job.addAndWait();
            if (_debugging) {            
                     _debug("Add job finished");
            }
            // Get the list of jobs from the response
            List jobs = jobResponse.getJobs();
            // We know there is only 1 job in the list since we made the call
            // to add the job
            job = (Job)jobs.get(0);
            // Print some information about the newly added job
            if (_debugging) {
                _debug("\n --------- Job Information--------- " + 
                    "\n - Name '" + job.getName() +
                    "'\n - Description '" + job.getDescription() + 
                    "'\n - Owner '" + job.getOwner() +
                    "'\n - Executable '" + job.getExecutable() +
                    "'\n - Parameters '" + job.getParameters() +
                    "'\n");
            }

            ////////////////////////////////////////////////////////////////////////////////////
            // Update the running status
            // This example uses polling to look for the status changes of the run.
            long start = System.currentTimeMillis();
            int job_timeout = ((IntToken)timeout.getToken()).intValue();
            
            // Submit the job for execution and wait for the response
            RunResponse runResponse = job.executeAndWait();
             // Get the run handler for the session
            RunHandler runHandler = session.getRunHandler();
            // Get the list of runs from the response
            List runs = runResponse.getRuns();
            // We know there is only 1
            Run run = (Run)runs.get(0);
            
  
            while (!"Succeeded".equals(run.getStatus()) && !"Failed".equals(run.getStatus())) {
                // 1 second pause
                Thread.sleep(1000);
                
                if(job_timeout > 0)
                {
                    // We'll only wait for time out setting
                    if ((System.currentTimeMillis() - start) > 60000*job_timeout) {
                        if (_debugging) {
                            _debug("Job " + job.getName() + " is canceled for time out." );
                        }
                        run.cancelAndWait();
                        break;
                    }
                }

                runResponse = runHandler.getRunAndWait(run.getID());
                runs = runResponse.getRuns();
                run = (Run)runs.get(0);

                // Watching the Run status
                outStr = "\n --------- Job Running status --------- " + 
                    "\n Name '" + run.getID() +
                        "'\n - Status '" + run.getStatus() +
                        "'\n - Start '" + run.getStartedDate() +
                        "'\n - Finish '" + run.getFinishedDate() +
                        "'\n - CPU used '" + run.getHoursUsed() +
                        "'\n - CPU billed '" + run.getHoursBilled() +
                        "'\n - Job Name '" + run.getJobName() +
                        "'\n";

                 if (_debugging) {
                    _debug(outStr);
                }
             }

            ////////////////////////////////////////////////////////////////////////////////////            
            // Download the run output and put it into the specified local file
            String file_name = ((StringToken)outputFile.getToken()).stringValue();
            File output_file = new File(file_name);
            if ( output_file.isDirectory() )
                file_name = output_file.getPath() + File.separator + run.getID() + ".output.zip" ;
                
            runResponse = run.downloadAndWait(file_name);
            outputResource.broadcast(new StringToken(output_file.getAbsolutePath())); 

            
            outStr = "Job '" + job.getName() + "' " + run.getStatus() + ".\n " + 
                    "\n See the output file '" + file_name + "' for detail infomation.\n";
            if (_debugging) {
                _debug(outStr);
            }

            outputMonitor.send(0, new StringToken(outStr));
            ////////////////////////////////////////////////////////////////////////////////////
            // Logout out of the GridPortal
            session.logoutAndWait();
        }
        catch (InterruptedException ex) {
            System.out.println("The exception raised the message " + ex.getMessage());  
            ex.printStackTrace();
        } catch (GridException ex) {
            System.out.println("The exception raised the message " + ex.getMessage());  
            ex.printStackTrace();
        }
        
     }

     public boolean prefire()
        throws IllegalActionException   
     {
         trustStore = System.getProperty("javax.net.ssl.trustStore");
         System.clearProperty("javax.net.ssl.trustStore");
         
         return super.prefire();
     }
    
    public boolean postfire()
        throws IllegalActionException   
    {
        if(null != session)
        {
            session.close();
            session = null;
        }
        
        System.setProperty("javax.net.ssl.trustStore", trustStore);
        return super.postfire();
    }
 
    public static class ResponseInfo
    {
        /** success/failure flag */
        private boolean _success = true;
    
        /** a textual description of the result */
        private String _description = "";
        
        public ResponseInfo(boolean success) 
        {
            _success = success;
        }
    
        public ResponseInfo(boolean success, String description) 
        {
            this(success);
            _description = description;
        }
    
        /**
         * Get a textual description.  The textual description is
         * normally only used for describing login failures.  
         *
         * @return the textual description of the result
         */ 
        public String getDescription()
        {
            return _description;
        }
    
        /**
         * Determine if the result was successfull.  If the result is not successfull
         * the getDescription() method can be called to get a textual description
         * of the reason for failure.<
         *
         * @return true if the login succeeded, false otherwise
         */
        public boolean isSuccessfull()
        {
            return _success;
        }
    
    }
}





