/*
 * SunGridInputResource.java
 *
 * Created on Dec.13, 2007, 4:23 PM, 
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.monash.sungrid;


import java.io.File;
import javax.swing.JOptionPane;
import ptolemy.actor.Director;
import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.gui.style.TextStyle;
import ptolemy.data.ArrayToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.FileParameter;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;
import ptolemy.kernel.util.Attribute;
import ptolemy.kernel.util.NameDuplicationException;
import ptolemy.kernel.util.IllegalActionException;

/**
  *   <p> SunGridInputResource.java is a subclass of ptolemie's TypedAtomicor.
  *   The actor is used to define a resource for Sun Grid, the defined resources will be uploaded to 
  *   Sun Grid when the related job is running.
  *   Sun Grid resource is composed by 
  *   1) Resource Name, which is used to identify resouces on SunGrid,
  *      It should be unique charactor for resource
  *   2) Resource Description
  *   3) Resource type, which may be one of the 
  *      * Data
  *      * Application
  *      * Library
  *      * Terms of Use Agreement
  *      * Digital Entitlement Token(DET)
  *      * X.509 Certificate
  *   4) Resource Notes
  *   5) Resource File, which is the most important information of Resource, 
  *      the defined file will be uploaded to Sun Grid, and the running is based on these files
  *      Resource files are commonly packaged in an archive format, that allows file system hierarchies 
  *      to be conveniently uploaded, such as .zip file
  *
  *   <p>To access Sun Grid(network.com), there are three important actors
  *     SunGridLogin            --  Setting Login info, which include user name and proxy 
  *     SunGridInputResource    --  Define resource files for Sun Grid job, and will be upload to Sun Grid before job running
  *                                 One Sun Grid job can link with several SunGridInputResource
  *     SunGridClient           --  Define Sun Grid job which is used to run application on Sun Grid</p>
  *
  *    
  * @author Chris Zhu, Sun Microsystems, Inc.
  * @version $Id: SunGridLogin.java, v 1.0 2007/12/13
  **/
public class SunGridInputResource extends TypedAtomicActor{

    // file input of SunGridInputResource,  
    // if resourceFilePort is linked, the Parameter resourceFile will be ignored
    public TypedIOPort resourceFilePort;    

    // resource output of SunGridInputResource, which will be the input for SunGridClient
    public TypedIOPort Resource;

    // resource name -- unique charactor for resource
    public Parameter resourceName;
    // resource description
    public Parameter resourceDescription;
    // resource type, which may be one of the 
    //      * Data
    //      * Application
    //      * Library
    //      * Terms of Use Agreement
    //      * Digital Entitlement Token(DET)
    //      * X.509 Certificate
    public Parameter resourceType;
    // resource notes
    public Parameter resourceNotes;
    // resource file
    // resource are commonly packaged in an archive format 
    // that allows file system hierarchies to be conveniently uploaded, 
    // such as .zip file
    public FileParameter resourceFile;

    /** Creates a new instance of SunGridInputResource */
    public SunGridInputResource(CompositeEntity container, String name)
    throws NameDuplicationException, IllegalActionException  
    {
        super(container, name);
        
        //------------- initialize output port -------------
        Resource = new TypedIOPort(this, "Resource", false, true);
        Resource.setTypeEquals(new ArrayType(BaseType.STRING));
        new Parameter(Resource, "_showName", BooleanToken.TRUE);
    
        //------------- initialize parameters -------------
        // resource Name  -- must be unique defined
        // can't be null
        resourceName = new Parameter(this, "Resource Name", new StringToken("Can't be null"));
        resourceName.setStringMode(true);
 
        // resourceDescription
        resourceDescription = new Parameter(this, "Resource Description", new StringToken(""));
	// set the default script
	resourceDescription.setExpression("Enter Description here...");
        resourceDescription.setStringMode(true);
        
        // resourceType   -- single choice, Type Data is the default choice
        resourceType = new Parameter(this, "Resource Type", new StringToken("Data"));
        resourceType.setStringMode(true);
        _addResourceTypeChoice();
      
        resourceNotes = new Parameter(this, "Resource Notes", new StringToken(""));
        // make a text style, so that the parameter has multiple lines
        new TextStyle(resourceNotes, "Resource Notes");
        // set the default script
	resourceNotes.setExpression("You can add your notes here...");
        resourceNotes.setStringMode(true);

        // resourceFile   -- file for upload 
        resourceFile = new FileParameter(this, "Resource File");

        // or resource File read from input port
        resourceFilePort = new TypedIOPort(this, "Resource File", true, false);
        resourceFilePort.setTypeEquals(BaseType.STRING);
        
	_attachText("_iconDescription", "<svg>\n"
			+ "<rect x=\"-25\" y=\"-20\" " + "width=\"50\" height=\"40\" "
			+ "style=\"fill:white\"/>\n"
			+ "<polygon points=\"-15,-10 -12,-10 -8,-14 -1,-14 3,-10"
			+ " 15,-10 15,10, -15,10\" " + "style=\"fill:yellow\"/>\n"
			+ "</svg>\n");        
    }
 
    private Token getParaArray(String filename) throws IllegalActionException 
    {
        Token retval = null;
        Token[] para = new Token[5];
        
        try
         {
             para[0] = resourceName.getToken();
             para[1] = resourceDescription.getToken();
             para[2] = resourceType.getToken();
             para[3] = resourceNotes.getToken();
             para[4] = new StringToken(filename);
          }
          catch (Exception ex) {
            System.out.println("The exception raised the message " + ex.getMessage());  
            ex.printStackTrace();
         }
        
         retval = new ArrayToken(para);
         return retval;
     }
    
    public synchronized void fire() throws IllegalActionException 
    {
        try
        {
            String errorMessage;
            String name = ((StringToken)resourceName.getToken()).stringValue().trim();
            
            if(name.isEmpty())
            {
                errorMessage = "Resource name is NULL\n Please define the resource Name";
            }
            else
            {
                // If the resourceFile input port is connected and has data, then
                // get the file name from there.
                if (resourceFilePort.getWidth() > 0) {
                    if (resourceFilePort.hasToken(0)) {
                        String filename = ((StringToken) resourceFilePort.get(0)).stringValue();

                        // Using setExpression() rather than setToken() allows
                        // the string to refer to variables defined in the
                        // scope of this actor.
                        resourceFile.setExpression(filename);
                    }
                }
                
                // get file name form parameter resourceFile
                File file = resourceFile.asFile();
            
                if(file.exists())//目录或文件是否存在
                {
                    if(file.isDirectory())
                    {
                        errorMessage = "Can't add a directory to resource ";
                    }
                    else
                    {
                        // send resource to SUnGridClient
                        Resource.send(0, getParaArray(file.getAbsolutePath()));
                        return;                    
                    }
                }
                else
                {
                    errorMessage = "Resource file don't exist " + file.getAbsolutePath();
                }
            }
                    
            JOptionPane.showMessageDialog(null, 
                            "Resource Error: " + name + "\n" + errorMessage, 
                            "Resource Error",
                            JOptionPane.ERROR_MESSAGE);
    
            Director director = getDirector();

            if (director != null) {
                director.stop();
            }
           // Resource.send(0, new StringToken(filePathStr));
            
        }
        catch (Exception ex) {
            System.out.println("The exception raised the message " + ex.getMessage());  
            ex.printStackTrace();
         }
        
    }
       
    public boolean postfire() {
        return false;
    }
    
  ////////////////// Private Methods /////////////////////// 
  //creates list for adding privider ...  
  private void _addResourceTypeChoice()
    {
        resourceType.addChoice("Data");
        resourceType.addChoice("Application");             
        resourceType.addChoice("Library");      
        resourceType.addChoice("Terms of Use Agreement");
        resourceType.addChoice("Digital Entitlement Token(DET)");  
        resourceType.addChoice("X.509 Certificate");
    }
  
 }




