/*
 * SunGridLogin.java
 *
 * Created on Feb.10, 2008, 11:31 AM, 
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 *
 */

package org.monash.sungrid;

import ptolemy.actor.TypedAtomicActor;
import ptolemy.actor.TypedIOPort;
import ptolemy.actor.gui.style.NotEditableLineStyle;
import ptolemy.data.ArrayToken;
import ptolemy.data.BooleanToken;
import ptolemy.data.StringToken;
import ptolemy.data.Token;
import ptolemy.data.expr.Parameter;
import ptolemy.data.type.ArrayType;
import ptolemy.data.type.BaseType;
import ptolemy.kernel.CompositeEntity;

import ptolemy.kernel.util.IllegalActionException;
import ptolemy.kernel.util.NameDuplicationException;

/**
  *   <p>SunGridLogin is a subclass of ptolemie's TypedAtomicor.
  *   The actor is used to define the login settings for Sun Grid connection,which include
  *   
  *   1) User Name
  *   2) Proxy settings, which include 
  *      * Proxy 
  *      * Port
  *      Set the proxy if it's necessary for your network access
  *      You can consult your network administrator for proxy setting.
  *   3) URL, which specify the Service location, (uneditable)
  *      The default value of url is "https://www.network.com/GridPortal/GridPortal"
  *   4) API settings, (uneditable)
  *      * Adapter Name  -- "gridclient.internalAPI" by default
  *      * Grid Class    -- "com.sun.sungrid.privateplugin.PluginConnection" by default </p>
  *
  *   <p>To access Sun Grid(network.com), there are three important actors
  *     SunGridLogin            --  Setting Login info, which include user name and proxy 
  *     SunGridInputResource    --  Define resource files for Sun Grid job, and will be upload to Sun Grid before job running
  *                                 One Sun Grid job can link with several SunGridInputResource
  *     SunGridClient           --  Define Sun Grid job which is used to run application on Sun Grid</p>
  *
  *    
  * @author Chris Zhu, Sun Microsystems, Inc.
  * @version $Id: SunGridLogin.java, v 1.0 2008/02/10
  **/
public class SunGridLogin  extends TypedAtomicActor {

    // delieve the login info to SunGrid 
    public TypedIOPort loginPort;
   
    public Parameter userName;
    public Parameter proxy;
    public Parameter port;
    public Parameter url;
    public Parameter adapterName;
    public Parameter gridClass;
    
    /** Creates a new instance of SunGridLogin */
    public SunGridLogin(CompositeEntity container, String name)
            throws NameDuplicationException, IllegalActionException  
    {
        super(container, name);
        
        //------------- initialize output port -------------
        loginPort = new TypedIOPort(this, "loginPort", false, true);
        loginPort.setTypeEquals(new ArrayType(BaseType.STRING));
        new Parameter(loginPort, "_showName", BooleanToken.TRUE);
        
        //------------- initialize parameters -------------
        userName = new Parameter(this, "User Name", new StringToken("A valid Sun Grid account id should be used!"));
        userName.setStringMode(true);
 
        proxy = new Parameter(this, "Proxy", new StringToken(""));
        proxy.setStringMode(true);

        port = new Parameter(this, "Port", new StringToken(""));
        port.setStringMode(true);

        
        url = new Parameter(this, "Sun Grid URL", new StringToken("https://www.network.com/GridPortal/GridPortal"));
        new NotEditableLineStyle(url,"Sun Grid URL" );
        url.setExpression("https://www.network.com/GridPortal/GridPortal");
        url.setStringMode(true);
        
        gridClass = new Parameter(this, "Grid Class", new StringToken("com.sun.sungrid.privateplugin.PluginConnection"));
        new NotEditableLineStyle(gridClass, "Grid Class");
        gridClass.setExpression("com.sun.sungrid.privateplugin.PluginConnection");
        gridClass.setStringMode(true);
       
        adapterName = new Parameter(this, "Adapter Name", new StringToken("gridclient.internalAPI"));
        new NotEditableLineStyle(adapterName, "Adapter Name");
        adapterName.setExpression("gridclient.internalAPI");
        adapterName.setStringMode(true);
 
  
	_attachText("_iconDescription", "<svg>\n"
			+ "<rect x=\"-25\" y=\"-20\" " + "width=\"50\" height=\"40\" "
			+ "style=\"fill:white\"/>\n"
			+ "<polygon points=\"-15,-10 -12,-10 -8,-14 -1,-14 3,-10"
			+ " 15,-10 15,10, -15,10\" " + "style=\"fill:yellow\"/>\n"
			+ "</svg>\n");   

      }

    private Token getParaArray() throws IllegalActionException 
    {
         Token retval = null;
         Token[] para = new Token[6];
         
         try
         {
             para[0] = userName.getToken();
             para[1] = proxy.getToken();
             para[2] = port.getToken();
             para[3] = url.getToken();
             para[4] = adapterName.getToken();
             para[5] = gridClass.getToken();
         }
          catch (Exception ex) {
            System.out.println("The exception raised the message " + ex.getMessage());  
            ex.printStackTrace();
        }
          
         retval = new ArrayToken(para);
         return retval;
    }

    public synchronized void fire() throws IllegalActionException 
    {
        try
        {
            loginPort.send(0, getParaArray());
        }
        catch (Exception ex) {
            System.out.println("The exception raised the message " + ex.getMessage());  
            ex.printStackTrace();
        }
    }
}





